package de.unikoblenz.fgbks.base.utils.boundary;

/**
 * The BoundType determines, if the border of a boundary is {@link BoundType#INCLUSIVE} or {@link
 * BoundType#EXCLUSIVE}.
 */
public enum BoundType {
  /**
   * The bound of the boundary is inclusive, means, that a value, which is exact at the bound, the
   * value is inside the boundary. <br> Example: <br> value = 10; boundary = {@literal <}=10 -{@literal >} {@link
   * BoundType} is INCLUSIVE. So the value is inside the boundary.
   */
  INCLUSIVE,
  /**
   * The bound of the boundary is exclusive, means, that a value, which is exact at the bound, the
   * value is outside the boundary. <br>
   * Example: <br>
   * value = 10; boundary = {@literal <}10 -{@literal >} {@link BoundType} is EXCLUSIVE. So the value is outside the
   * boundary.
   */
  EXCLUSIVE;

  /**
   * Get the opposite of the {@link BoundType}.
   *
   * @return {@link BoundType#INCLUSIVE}, if {@code other} is {@link BoundType#EXCLUSIVE}, else
   *     {@link BoundType#EXCLUSIVE}
   */
  public static BoundType getOp(BoundType other) {
    return other.getOp();
  }

  /**
   * Get the opposite of the {@link BoundType}.
   *
   * @return {@link BoundType#INCLUSIVE}, if current is {@link BoundType#EXCLUSIVE}, else {@link
   * BoundType#EXCLUSIVE}
   */
  public BoundType getOp() {
    return this == INCLUSIVE ? EXCLUSIVE : INCLUSIVE;
  }
}
