package de.unikoblenz.fgbks.base.utils.boundary;

import de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreater;
import de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType;
import de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheck;
import de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType;
import de.unikoblenz.fgbks.base.utils.boundary.creater.BoundaryCreater;
import de.unikoblenz.fgbks.base.utils.boundary.creater.BoundaryCreaterType;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import java.util.Optional;

/**
 * A boundary express a set of values, which are in a given range or between given bounds.
 */
public interface Boundary extends Comparable<Boundary> {

  /**
   * Get the {@link VTypeRef} of the current boundary.
   *
   * @return the typeRef
   */
  VTypeRef getTypeRef();

  /**
   * Get the original text as string the boundary was parsed from.
   *
   * @return the original text as string
   */
  String getText();

  /**
   * Get the text as string, representing the boundary.
   *
   * @return the boundary as text representation
   */
  String getParsedText();

  /**
   * Perform a {@link BoundaryCheck} operation with an other boundary. The boundaries should be from
   * the same type. All possible checks are listed in {@link BoundaryCheckType}.
   *
   * @param checkType a {@link BoundaryCheckType}
   * @param other the other Boundary
   * @return true, if the check returns true
   */
  boolean checkWith(BoundaryCheckType checkType, Boundary other);

  /**
   * Perform a {@link BoundaryCreater} operation, which creates a new boundary from the given one.
   * All possible checks are listed in {@link BoundaryCreaterType}.
   *
   * @param createType a {@link BoundaryCreaterType}
   * @return a optional with a new boundary with new bounds created by the {@link BoundaryCreater}.
   *     The optional is empty, if no new boundary could be created.
   */
  Optional<Boundary> create(BoundaryCreaterType createType);

  /**
   * Perform a {@link BoundaryBiCreater} operation, which creates a new boundary from the given one
   * and an {@code other} boundary. Both boundary should be from the same boundary type. All
   * possible checks are listed in {@link BoundaryBiCreaterType}.
   *
   * @param biCreaterType a {@link BoundaryCreaterType}
   * @param other the other boundary
   * @return a optional with a new boundary with new bounds created by the {@link
   * BoundaryBiCreater}. The optional is empty, if no new boundary could be created.
   */
  Optional<Boundary> createBi(BoundaryBiCreaterType biCreaterType, Boundary other);
}
