package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_EQUAL;
import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_NOT_IN_CONTACT;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

/**
 * The lower bound creator implementation of {@link BoundaryBiCreaterType#LOWER_BOUNDS} for {@link
 * StringBoundary}. <br> The result is a new boundary with the lower bounds, or an empty optional,
 * if the creation is not possible. <br> In: {@code b1: "a","b"; b2: "a"}<br> Out: {@code "b"} <br>
 */
public class BiCreaterStringLowerBounds extends AbstractBoundaryBiCreater<StringBoundary> {

  private static final BiCreaterStringLowerBounds instance = new BiCreaterStringLowerBounds();

  private BiCreaterStringLowerBounds() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static BiCreaterStringLowerBounds getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return BoundaryBiCreaterType.LOWER_BOUNDS;
  }

  @Override
  public Optional<StringBoundary> create(StringBoundary b1, StringBoundary b2) {
    if (b1.checkWith(IS_NOT_IN_CONTACT, b2) || b1.checkWith(IS_EQUAL, b2)) {
      return Optional.empty();
    }
    if (b1.matchesAny() || b1.matchesNoneOfValues()) {
      return Optional.of(
          StringBoundary.getBuilder()
              .addValues(b1.getValues())
              .addValues(b2.getValues())
              .matchesNoneOfValues(!b2.matchesNoneOfValues())
              .build());
    }
    Set<String> values = new HashSet<>(Arrays.asList(b1.getValues()));
    values.removeAll(Arrays.asList(b2.getValues()));
    return Optional.of(StringBoundary.getBuilder().addValues(values).build());
  }
}
