package de.unikoblenz.fgbks.base.utils.boundary.checker;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;

/**
 * The equal checker implementation of {@link BoundaryCheckType#IS_EQUAL} for {@link
 * AbstractGrowingBoundary}. <br> The check returns true, if the first boundary is equal to the
 * other boundary. <br> E.g: <br>
 *
 * <ul>
 *   <li>b1 = "= 1"; b2 = "= 1" -{@literal >} true
 *   <li>b1 = "{@literal <}= 1"; b2 = "{@literal <} 2" -{@literal >} true (Integer or Long) / false (others)
 *   <li>b1 = "[0..20]"; b2 = "[4..22]" -{@literal >} false
 * </ul>
 */
public class CheckEqual extends AbstractBoundaryCheck<AbstractGrowingBoundary> {

  private static final CheckEqual instance = new CheckEqual();

  private CheckEqual() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CheckEqual getInstance() {
    return instance;
  }

  @Override
  public BoundaryCheckType getType() {
    return BoundaryCheckType.IS_EQUAL;
  }

  @Override
  public boolean check(AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    return checkEqual(b1, b2);
  }

  /** see {@link CheckEqual} */
  public static boolean checkEqual(AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    if (b1 == b2) {
      return true;
    }
    return ((b1.getLowerBound().equals(b2.getLowerBound())
        && b1.getLowerBoundType().equals(b2.getLowerBoundType())
        && b1.getUpperBound().equals(b2.getUpperBound())
        && b1.getUpperBoundType().equals(b2.getUpperBoundType())));
  }
}
