package de.unikoblenz.fgbks.core.dmn.domain.vdmn;

import de.unikoblenz.fgbks.core.dmn.domain.common.RowNumber;
import de.unikoblenz.fgbks.core.dmn.domain.ids.RuleId;
import java.util.List;

/**
 * This interface represents a single rule (or row) in a {@link VDmnDecisionTable}. A rule contains
 * a list of {@link VDmnInputValue}s and a list of {@link VDmnOutputValue}s.
 */
public interface VDmnRule extends VDmnElement {

  /**
   * Get the {@link RowNumber} of the rule inside the dmn table.
   *
   * @return the {@link RowNumber}.
   */
  RowNumber getRowNumber();

  /**
   * Get the {@link VDmnDecisionTable}.
   *
   * @return the {@link VDmnDecisionTable}.
   */
  VDmnDecisionTable getDmnDecisionTable();

  /**
   * Get a list of {@link VDmnInputValue}s.
   *
   * @return the list of {@link VDmnInputValue}
   */
  List<VDmnInputValue> getDmnInputValues();

  /**
   * Get a list of {@link VDmnOutputValue}s.
   *
   * @return the list of {@link VDmnOutputValue}
   */
  List<VDmnOutputValue> getDmnOutputValues();

  /**
   * Get the {@link RuleId} of the {@link VDmnRule}.
   *
   * @return the {@link RuleId}.
   */
  default RuleId getRuleId() {
    return (RuleId) getId();
  }

  /**
   * Get the {@link VDmnDecision}.
   *
   * @return the {@link VDmnDecision}.
   */
  default VDmnDecision getDmnDecision() {
    return getDmnDecisionTable().getDmnDecision();
  }

  /**
   * Get the {@link VDmnDefinition}.
   *
   * @return the {@link VDmnDefinition}.
   */
  default VDmnDefinition getDmnDefinition() {
    return getDmnDecision().getDmnDefinition();
  }
}
