package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.base.domain.Label;
import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.base.domain.StringValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

/**
 * Implementation of {@link VDmnColumn}
 */
public abstract class AbstractVDmnColumnImpl extends AbstractVDmnElement implements VDmnColumn {

  protected VDmnDecisionTable dmnDecisionTable;
  protected VTypeRef typeRef;
  protected List<VDmnValue> values;
  protected List<StringValue> predefinedValues;
  protected Name name;
  protected Label label;

  protected AbstractVDmnColumnImpl() {
    values = new ArrayList<>();
    predefinedValues = new ArrayList<>();
  }

  @Override
  public VDmnDecisionTable getDmnDecisionTable() {
    return dmnDecisionTable;
  }

  @Override
  public Optional<List<StringValue>> getPredefinedValues() {
    if (!getTypeRef().equals(VTypeRef.STRING) || predefinedValues.size() == 0) {
      return Optional.empty();
    }
    return Optional.of(Collections.unmodifiableList(predefinedValues));
  }

  @Override
  public VTypeRef getTypeRef() {
    return typeRef;
  }

  @Override
  public List<VDmnValue> getValues() {
    return Collections.unmodifiableList(values);
  }

  @Override
  public Optional<Name> getName() {
    return Optional.ofNullable(name);
  }

  @Override
  public Optional<Label> getLabel() {
    return Optional.ofNullable(label);
  }
}
