package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.core.dmn.domain.ids.DecisionTableId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.apache.commons.lang3.Validate;
import org.camunda.bpm.model.dmn.HitPolicy;

/**
 * Implementation of {@link VDmnDecisionTable}
 */
public class VDmnDecisionTableImpl extends AbstractVDmnElement implements VDmnDecisionTable {

  private VDmnDecision dmnDecision;
  private HitPolicy hitPolicy;
  private List<VDmnOutputColumn> dmnOutputColumns;
  private List<VDmnInputColumn> dmnInputColumns;
  private List<VDmnRule> dmnRules;

  private VDmnDecisionTableImpl() {
    super();
    dmnOutputColumns = new ArrayList<>();
    dmnInputColumns = new ArrayList<>();
    dmnRules = new ArrayList<>();
  }

  @Override
  public VDmnDecision getDmnDecision() {
    return dmnDecision;
  }

  @Override
  public HitPolicy getHitPolicy() {
    return hitPolicy;
  }

  @Override
  public List<VDmnOutputColumn> getOutputColumns() {
    return Collections.unmodifiableList(dmnOutputColumns);
  }

  @Override
  public List<VDmnInputColumn> getInputColumns() {
    return Collections.unmodifiableList(dmnInputColumns);
  }

  @Override
  public List<VDmnRule> getRules() {
    return Collections.unmodifiableList(dmnRules);
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnDecisionTableImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnDecisionTableImpl> {

    public Builder withDecisionTableId(DecisionTableId decisionTableId) {
      value.id = decisionTableId;
      return this;
    }

    public Builder withDmnDecision(VDmnDecision dmnDecision) {
      value.dmnDecision = dmnDecision;
      return this;
    }

    public Builder withHitPolicy(HitPolicy hitPolicy) {
      value.hitPolicy = hitPolicy;
      return this;
    }

    public Builder addDmnOutputColumn(VDmnOutputColumn outputColumn) {
      value.dmnOutputColumns.add(Validate.notNull(outputColumn));
      return this;
    }

    public Builder addDmnOutputColumnFromBuilder(VDmnOutputColumnImpl.Builder outputColumnBuilder) {
      addDmnOutputColumn(Validate.notNull(outputColumnBuilder.getUnbuildValue()));
      outputColumnBuilder.withDecisionTable(value);
      return this;
    }

    public Builder addDmnInputColumn(VDmnInputColumn inputColumn) {
      value.dmnInputColumns.add(Validate.notNull(inputColumn));
      return this;
    }

    public Builder addDmnInputColumnFromBuilder(VDmnInputColumnImpl.Builder inputColumnBuilder) {
      addDmnInputColumn(Validate.notNull(inputColumnBuilder.getUnbuildValue()));
      inputColumnBuilder.withDecisionTable(value);
      return this;
    }

    public Builder addRule(VDmnRule dmnRule) {
      value.dmnRules.add(Validate.notNull(dmnRule));
      return this;
    }

    public Builder addRuleFromBuilder(VDmnRuleImpl.Builder dmnRuleBuilder) {
      addRule(Validate.notNull(dmnRuleBuilder.getUnbuildValue()));
      dmnRuleBuilder.withDmnDecisionTable(value);
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.dmnDecision);
      Validate.notNull(value.hitPolicy);
    }
  }
}
