package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.base.domain.Label;
import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.base.domain.StringValue;
import de.unikoblenz.fgbks.core.dmn.domain.ids.InputExpressionId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.InputId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import org.apache.commons.lang3.Validate;

/**
 * Implementation of {@link VDmnInputColumn}.
 */
public class VDmnInputColumnImpl extends AbstractVDmnColumnImpl implements VDmnInputColumn {

  private InputExpressionId inputExpressionId;

  private VDmnInputColumnImpl() {
    super();
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnInputColumnImpl().new Builder();
  }

  @Override
  public InputExpressionId getInputExpressionId() {
    return inputExpressionId;
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnInputColumnImpl> {

    public Builder withInputId(InputId inputId) {
      value.id = inputId;
      return this;
    }

    public Builder withDecisionTable(VDmnDecisionTable decisionTable) {
      value.dmnDecisionTable = decisionTable;
      return this;
    }

    public Builder withInputExpressionId(InputExpressionId inputExpressionId) {
      value.inputExpressionId = inputExpressionId;
      return this;
    }

    public Builder withTypeRef(VTypeRef typeRef) {
      value.typeRef = typeRef;
      return this;
    }

    public Builder withName(String name) {
      value.name = new Name(name);
      return this;
    }

    public Builder withLabel(String label) {
      value.label = new Label(label);
      return this;
    }

    public Builder addValue(VDmnInputValue dmnValue) {
      value.values.add(Validate.notNull(dmnValue));
      return this;
    }

    public Builder addValueFromBuilder(VDmnInputValueImpl.Builder dmnValueBuilder) {
      addValue(Validate.notNull(dmnValueBuilder.getUnbuildValue()));
      dmnValueBuilder.withDmnInputColumn(value);
      return this;
    }

    public Builder addPredefinedInputValue(String predefinedValue) {
      value.predefinedValues.add(new StringValue(predefinedValue));
      return this;
    }

    public Builder addPredefinedInputValues(String[] predefinedValues) {
      Validate.noNullElements(predefinedValues);
      for (String s : predefinedValues) {
        addPredefinedInputValue(s);
      }
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.typeRef);
      Validate.notNull(value.inputExpressionId);
      Validate.notNull(value.dmnDecisionTable);
    }
  }
}
