package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.base.domain.Description;
import de.unikoblenz.fgbks.base.domain.Text;
import de.unikoblenz.fgbks.base.utils.boundary.Boundary;
import de.unikoblenz.fgbks.core.dmn.domain.ids.InputEntryId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import org.apache.commons.lang3.Validate;

/**
 * Implementation of {@link VDmnInputValue}.
 */
public class VDmnInputValueImpl extends AbstractVDmnValue implements VDmnInputValue {

  protected Boundary boundary;

  protected VDmnInputValueImpl() {
    super();
  }

  @Override
  public Boundary getBoundary() {
    return boundary;
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnInputValueImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnInputValueImpl> {

    public Builder withDmnRule(VDmnRule dmnRule) {
      value.dmnRule = dmnRule;
      return this;
    }

    public Builder withDmnRuleFromBuilder(VDmnRuleImpl.Builder dmnRuleBuilder) {
      value.dmnRule = dmnRuleBuilder.getUnbuildValue();
      return this;
    }

    public Builder withText(String text) {
      value.text = new Text(text);
      return this;
    }

    public Builder withDescription(String description) {
      value.description = new Description(description);
      return this;
    }

    public Builder withDmnInputColumn(VDmnInputColumn dmnColumn) {
      value.dmnColumn = dmnColumn;
      return this;
    }

    public Builder withInputEntryId(InputEntryId inputEntryId) {
      value.id = inputEntryId;
      return this;
    }

    public Builder withBoundary(Boundary boundary) {
      value.boundary = boundary;
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.dmnRule);
      Validate.notNull(value.text);
      Validate.notNull(value.dmnColumn);
      Validate.notNull(value.id);
    }

    @Override
    public VDmnInputValueImpl build() {
      // create boundary, if not created before
      if (value.boundary == null) {
        value.boundary =
            value.dmnColumn.getTypeRef().getBoundaryFromText(value.getText().getValue());
      }
      return super.build();
    }
  }
}
