package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.core.dmn.domain.common.RowNumber;
import de.unikoblenz.fgbks.core.dmn.domain.ids.RuleId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.apache.commons.lang3.Validate;

/**
 * Implementation of {@link VDmnRule}.
 */
public class VDmnRuleImpl extends AbstractVDmnElement implements VDmnRule {

  protected RowNumber rowNumber;
  protected VDmnDecisionTable dmnDecisionTable;
  protected List<VDmnInputValue> inputValues;
  protected List<VDmnOutputValue> outputValues;

  protected VDmnRuleImpl() {
    inputValues = new ArrayList<>(10);
    outputValues = new ArrayList<>(3);
  }

  @Override
  public RowNumber getRowNumber() {
    return rowNumber;
  }

  @Override
  public VDmnDecisionTable getDmnDecisionTable() {
    return dmnDecisionTable;
  }

  @Override
  public List<VDmnInputValue> getDmnInputValues() {
    return Collections.unmodifiableList(inputValues);
  }

  @Override
  public List<VDmnOutputValue> getDmnOutputValues() {
    return Collections.unmodifiableList(outputValues);
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnRuleImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnRuleImpl> {

    public Builder withRowNumber(RowNumber rowNumber) {
      value.rowNumber = rowNumber;
      return this;
    }

    public Builder withRuleId(RuleId ruleId) {
      value.id = ruleId;
      return this;
    }

    public Builder withDmnDecisionTable(VDmnDecisionTable vDmnDecisionTable) {
      value.dmnDecisionTable = vDmnDecisionTable;
      return this;
    }

    public Builder addInputValue(VDmnInputValue inputValue) {
      value.inputValues.add(Validate.notNull(inputValue));
      return this;
    }

    public Builder addInputValueFromBuilder(VDmnInputValueImpl.Builder inputValueBuilder) {
      addInputValue(Validate.notNull(inputValueBuilder.getUnbuildValue()));
      inputValueBuilder.withDmnRule(value);
      return this;
    }

    public Builder addOutputValue(VDmnOutputValue outputValue) {
      value.outputValues.add(Validate.notNull(outputValue));
      return this;
    }

    public Builder addOutputValueFromBuilder(VDmnOutputValueImpl.Builder outputValueBuilder) {
      addOutputValue(Validate.notNull(outputValueBuilder.getUnbuildValue()));
      outputValueBuilder.withDmnRule(value);
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.rowNumber);
      Validate.notNull(value.dmnDecisionTable);
    }
  }
}
