package de.unikoblenz.fgbks.core.dmn.verification.result;

import de.unikoblenz.fgbks.base.utils.UniqueIdGenerator;
import java.io.Serializable;
import java.util.Objects;
import javax.json.bind.annotation.JsonbProperty;

/**
 * A Abstract Result object is a container for other parts for the result elements. All result
 * elements have a unique id and are serializable.
 */
public abstract class AbstractResultObject implements Serializable {

  private long id = UniqueIdGenerator.getNextId();

  /**
   * Get the unique id of the result element.
   *
   * @return the unique id
   */
  @JsonbProperty("id")
  public long getId() {
    return id;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    AbstractResultObject that = (AbstractResultObject) o;
    return id == that.id;
  }

  @Override
  public int hashCode() {
    return Objects.hash(id);
  }
}
