package de.unikoblenz.fgbks.core.dmn.verification.result;

import de.unikoblenz.fgbks.base.builder.DefaultBuilder;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import javax.json.bind.annotation.JsonbProperty;
import org.apache.commons.lang3.Validate;

/**
 * The verifier result set contains a set of {@link VerifierResult}s.
 */
public class VerifierResultSet extends AbstractResultObject {

  private Set<VerifierResult> verifierResults;

  /** Create a new instance of the VerifierResultSet. */
  protected VerifierResultSet() {
    this.verifierResults = ConcurrentHashMap.newKeySet();
  }

  /**
   * Get a new instance of a builder for a {@link VerifierResultSet}. The {@link VerifierResult}s
   * can be added async with {@link Builder#addVerifierResult(VerifierResult)}.
   *
   * @return a new {@link Builder}
   */
  public static Builder getBuilder() {
    return new VerifierResultSet().new Builder();
  }

  /**
   * Get the set verifier results.
   *
   * @return the set of {@link VerifierResult}s
   */
  @JsonbProperty("verifier")
  public Set<VerifierResult> getVerifierResults() {
    return new HashSet<>(verifierResults);
  }

  /**
   * Get the amount of verifier results, which are currently in the set.
   *
   * @return the amount of verifier results
   */
  @JsonbProperty("size")
  public int getAmountOfVerifier() {
    return verifierResults.size();
  }

  /**
   * A builder class for a {@link VerifierResultSet}. Add new {@link VerifierResult}s async with
   * {@link Builder#addVerifierResult(VerifierResult)}.
   */
  public class Builder extends DefaultBuilder<VerifierResultSet> {

    /**
     * Add a new {@link VerifierResult} to the set. This method can be called async.
     *
     * @param verifierResult the {@link VerifierResult}
     * @return the current {@link Builder}
     */
    public Builder addVerifierResult(VerifierResult verifierResult) {
      value.verifierResults.add(Validate.notNull(verifierResult));
      return this;
    }
  }
}
