package de.unikoblenz.fgbks.core.dmn.verification.verifier;

import de.unikoblenz.fgbks.core.dmn.verification.result.VerifierResult;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.VerificationType;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

/**
 * Interface for a DMN Verifier. The important function is the method {@link
 * Verifier#verify(ExecutorService)}. This interface is used by the * {@link AbstractVerifier}.
 */
public interface Verifier {

  /**
   * Get the {@link VerificationType} of the verifier.
   *
   * @return the {@link VerificationType}
   */
  VerificationType getVerificationType();

  /**
   * Generate a {@link VerifierResult} in a {@link Future} result.
   *
   * @param executor as {@link ExecutorService}
   * @return a {@link VerifierResult}
   */
  Future<VerifierResult> verify(ExecutorService executor);

  /**
   * This methods returns he execution time after the verifier finished the verification.
   *
   * @return the execution time in ms
   */
  long getExecutionTime();
}
