package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_EQUAL;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_RULES;

import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.IdenticalBusinessRuleVerification;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@DmnVerifier(verifierType = IdenticalBusinessRuleVerification.class)
public class IdenticalVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    for (VDmnDecision decision : dmnObjectContainer.getVDmnDefinition().getDmnDecisions()) {
      checkDecisionTable(decision.getDmnDecisionTable());
    }
  }

  private void checkDecisionTable(VDmnDecisionTable dmnDecisionTable) {
    checkForIdenticalRules(
        dmnDecisionTable, dmnDecisionTable.getInputColumns(), 0, dmnDecisionTable.getRules());
  }

  protected void checkForIdenticalRules(
      VDmnDecisionTable dmnDecisionTable,
      List<VDmnInputColumn> inputs,
      int i,
      List<VDmnRule> currentRules) {
    if (i == inputs.size()) {
      currentRules.forEach(
          rule ->
              vreFactory.addElement(
                  VerificationResultEntryElement.create(dmnDecisionTable)
                      .withIdentifier(rule.getRuleId())));
      vreFactory.addVerificationFix(SHOW_RULES);
      vreFactory.addToEntry(VerificationClassification.WARNING, "identical");
    } else {
      List<VDmnInputValue> curInVals = new ArrayList<>();
      List<VDmnInputValue> sortInVals =
          VDmnFunctions.getColumnValuesInRules(inputs.get(i), currentRules);
      sortInVals.sort(Comparator.comparing(VDmnInputValue::getBoundary));
      VDmnInputValue lastVal = null;
      for (VDmnInputValue curVal : sortInVals) {
        if (lastVal != null) {
          if (curVal.getBoundary().checkWith(IS_EQUAL, lastVal.getBoundary())) {
            curInVals.add(lastVal);
          } else {
            if (curInVals.size() > 0) {
              curInVals.add(lastVal);
              checkForIdenticalRules(
                  dmnDecisionTable,
                  inputs,
                  i + 1,
                  curInVals.stream().map(VDmnValue::getDmnRule).collect(Collectors.toList()));
            }
            curInVals.clear();
          }
        }
        lastVal = curVal;
      }
      if (curInVals.size() > 0) {
        curInVals.add(lastVal);
        checkForIdenticalRules(
            dmnDecisionTable,
            inputs,
            i + 1,
            curInVals.stream().map(VDmnValue::getDmnRule).collect(Collectors.toList()));
      }
    }
  }
}
