package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_INPUT_ENTRIES;

import de.unikoblenz.fgbks.base.utils.boundary.impl.InvalidBoundary;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.InputValueSyntaxVerification;

@DmnVerifier(verifierType = InputValueSyntaxVerification.class)
public class InputValueSyntaxVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    dmnObjectContainer.getVDmnDefinition().getDmnDecisions().stream()
        .flatMap(
            d ->
                d.getDmnDecisionTable().getRules().stream()
                    .flatMap(r -> r.getDmnInputValues().stream()))
        .forEach(this::check);
  }

  private void check(VDmnInputValue inputValue) {
    // no Date, because of extra DateVerifier
    if (inputValue.getDmnColumn().getTypeRef() != VTypeRef.DATE
        && inputValue.getBoundary() instanceof InvalidBoundary) {
      vreFactory
          .addElement(VerificationResultEntryElement.create(inputValue))
          .addVerificationFix(SHOW_INPUT_ENTRIES)
          .addToEntry(
              VerificationClassification.FATAL_ERROR,
              "Input value is not valid: " + inputValue.getText());
      System.out.println(inputValue.getText() + " IS INVALID SYNTAX");
    }
  }
}
