package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType.COMBINE;
import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_EQUAL;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_RULES;

import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.PartialReductionVerification;
import java.util.ArrayList;
import java.util.List;

@DmnVerifier(verifierType = PartialReductionVerification.class)
public class PartialReductionVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    for (VDmnDecision decision : dmnObjectContainer.getVDmnDefinition().getDmnDecisions()) {
      findPartialReduction(decision.getDmnDecisionTable());
    }
  }

  private void findPartialReduction(VDmnDecisionTable dmnDecisionTable) {
    List<List<VDmnRule>> identicalOutputCluster =
        VDmnFunctions.getRuleClustersWithIdenticalOutput(dmnDecisionTable);
    List<VDmnInputColumn> inColumns = dmnDecisionTable.getInputColumns();
    identicalOutputCluster.forEach(c -> findPartialReduction(inColumns, 0, c, false));
  }

  private void findPartialReduction(
      List<VDmnInputColumn> inColumns, int i, List<VDmnRule> clusterRules, boolean hasCombination) {
    if (i == inColumns.size()) {
      if (hasCombination) {
        clusterRules.forEach(
            rule ->
                vreFactory.addElement(
                    VerificationResultEntryElement.create(inColumns.get(0).getDmnDecisionTable())
                        .withIdentifier(rule.getRuleId())));
        vreFactory.addVerificationFix(SHOW_RULES);
        vreFactory.addToEntry(VerificationClassification.INFO, "PartialReduction");
      }
    } else {
      List<VDmnInputValue> rules =
          VDmnFunctions.getColumnValuesInRules(inColumns.get(i), clusterRules);
      List<List<VDmnRule>> nClusters = new ArrayList<>();
      List<Boolean> combinationCluster = new ArrayList<>();
      for (int i1 = 0; i1 < rules.size(); i1++) {
        VDmnInputValue currentRule = rules.get(i1);
        // 1. Find combinations
        if (!hasCombination) {
          for (int i2 = i1 + 1; i2 < rules.size(); i2++) {
            VDmnInputValue currentRuleInner = rules.get(i2);
            if (currentRule != currentRuleInner) {
              if (currentRule
                  .getBoundary()
                  .createBi(COMBINE, currentRuleInner.getBoundary())
                  .isPresent()) {
                List<VDmnRule> cluster = new ArrayList<>();
                cluster.add(currentRule.getDmnRule());
                cluster.add(currentRuleInner.getDmnRule());
                nClusters.add(cluster);
                combinationCluster.add(true);
              }
            }
          }
        }
        // 2. Find equal values
        for (int i2 = i1 + 1; i2 < rules.size(); i2++) {
          VDmnInputValue currentRuleInner = rules.get(i2);
          if (currentRule != currentRuleInner) {
            if (currentRule.getBoundary().checkWith(IS_EQUAL, currentRuleInner.getBoundary())) {
              List<VDmnRule> cluster = new ArrayList<>();
              cluster.add(currentRule.getDmnRule());
              cluster.add(currentRuleInner.getDmnRule());
              nClusters.add(cluster);
              combinationCluster.add(false);
            }
          }
        }
      }
      // nex col
      for (int x = 0; x < combinationCluster.size(); x++) {
        if (nClusters.get(x).size() > 1) {
          findPartialReduction(
              inColumns, i + 1, nClusters.get(x), combinationCluster.get(x) || hasCombination);
        }
      }
    }
  }
}
