package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_INPUT_ENTRIES;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_OUTPUT_ENTRIES;

import de.unikoblenz.fgbks.base.domain.StringValue;
import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.PredefinedExistingValueVerification;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;

@DmnVerifier(verifierType = PredefinedExistingValueVerification.class)
public class PredefinedExistingValueVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    for (VDmnColumn stringColumn :
        VDmnFunctions.getColumnsByTypeRef(
            dmnObjectContainer.getVDmnDefinition(), VTypeRef.STRING)) {
      checkStringColumns(stringColumn);
    }
  }

  private void checkStringColumns(VDmnColumn stringColumn) {
    // Check, if there are  PredefinedValues defined
    boolean predefinedValuesArePresent = stringColumn.getPredefinedValues().isPresent();
    Set<String> predefinedValues = new HashSet<>();
    if (predefinedValuesArePresent) {
      predefinedValues =
          stringColumn.getPredefinedValues().get().stream()
              .map(StringValue::getValue)
              .collect(Collectors.toSet());
    }
    for (VDmnValue stringValue : stringColumn.getValues()) {
      // do not check empty values
      if (!stringValue.getText().getValue().isEmpty()) {
        if (stringValue.isInputValue()) {
          checkStringValue((VDmnInputValue) stringValue, predefinedValues);
        } else {
          checkStringValue((VDmnOutputValue) stringValue, predefinedValues);
        }
      }
    }
  }

  private void checkStringValue(VDmnOutputValue stringValue, Set<String> predefinedValues) {
    // get the Text and remove the "
    String parsedOutputText = stringValue.getText().getValue().replace('"', '\0').trim();
    if (!predefinedValues.contains(parsedOutputText)) {
      addToResult(stringValue, parsedOutputText);
    }
  }

  private void checkStringValue(VDmnInputValue stringValue, Set<String> predefinedValues) {
    // Do check with input boundary
    if (stringValue.getBoundary() instanceof StringBoundary) {
      StringBoundary sb = (StringBoundary) stringValue.getBoundary();
      for (String s : sb.getValues()) {
        if (!predefinedValues.contains(s)) {
          addToResult(stringValue, s);
        }
      }
    }
  }

  private void addToResult(VDmnValue stringValue, String missingStringValue) {
    vreFactory.addElement(VerificationResultEntryElement.create(stringValue));
    vreFactory.addVerificationFix(
        stringValue.isInputValue() ? SHOW_INPUT_ENTRIES : SHOW_OUTPUT_ENTRIES);
    vreFactory.addToEntry(
        VerificationClassification.WARNING,
        "String value \"%s\" was not found in the list of predefined values.",
        missingStringValue);
  }
}
