package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_INPUT_COLUMNS;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_OUTPUT_COLUMNS;

import de.unikoblenz.fgbks.base.domain.StringValue;
import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.PredefinedMissingValueVerification;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;

@DmnVerifier(verifierType = PredefinedMissingValueVerification.class)
public class PredefinedMissingValueVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    for (VDmnColumn stringColumn :
        VDmnFunctions.getColumnsByTypeRef(
            dmnObjectContainer.getVDmnDefinition(), VTypeRef.STRING)) {
      checkStringColumns(stringColumn);
    }
  }

  private void checkStringColumns(VDmnColumn stringColumn) {
    // Check, if there are  PredefinedValues defined
    boolean predefinedValuesArePresent = stringColumn.getPredefinedValues().isPresent();
    Set<String> predefinedValues = new HashSet<>();
    if (predefinedValuesArePresent) {
      predefinedValues =
          stringColumn.getPredefinedValues().get().stream()
              .map(StringValue::getValue)
              .collect(Collectors.toSet());
    }
    // Add all string values from the entries
    Set<String> stringValuesInColumn = new HashSet<>();
    for (VDmnValue stringValue : stringColumn.getValues()) {
      // do not check empty values
      if (!stringValue.getText().getValue().isEmpty()) {
        if (stringValue.isInputValue()) {
          stringValuesInColumn.addAll(getStringValues((VDmnInputValue) stringValue));
        } else {
          stringValuesInColumn.addAll(getStringValues((VDmnOutputValue) stringValue));
        }
      }
    }
    // get all missing strings
    Set<String> missingStrings = new HashSet<>();
    for (String s : stringValuesInColumn) {
      if (!predefinedValues.contains(s)) {
        missingStrings.add(s);
      }
    }
    if (missingStrings.size() > 0) {
      addToResult(stringColumn, missingStrings);
    }
  }

  private Set<String> getStringValues(VDmnOutputValue stringValue) {
    String parsedOutputText = stringValue.getText().getValue().replace('"', '\0').trim();
    return Collections.singleton(parsedOutputText);
  }

  private Set<String> getStringValues(VDmnInputValue stringValue) {
    if (stringValue.getBoundary() instanceof StringBoundary) {
      StringBoundary sb = (StringBoundary) stringValue.getBoundary();
      return new HashSet<>(Arrays.asList(sb.getValues()));
    }
    return Collections.emptySet();
  }

  private void addToResult(VDmnColumn stringColumn, Set<String> missingStringValues) {
    vreFactory.addElement(VerificationResultEntryElement.create(stringColumn));
    vreFactory.addVerificationFix(
        stringColumn.isInputColumn() ? SHOW_INPUT_COLUMNS : SHOW_OUTPUT_COLUMNS);
    StringBuilder sb = new StringBuilder();
    sb.append(missingStringValues.size() > 1 ? "These string values are" : "This string value is");
    sb.append(" not present in the predefined values of the column: ");
    sb.append(
        missingStringValues.stream().map(v -> '"' + v + '"').collect(Collectors.joining(",")));
    vreFactory.addToEntry(VerificationClassification.WARNING, sb.toString());
  }
}
