package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType.BETWEEN;
import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import io.quarkus.test.junit.QuarkusTest;
import org.junit.jupiter.api.Test;

@QuarkusTest
class BiCreaterStringBetweenTest {

  StringBoundary sb1;
  StringBoundary sb2;
  StringBoundary sbR;

  @Test
  void testCreater() {
    sb1 = StringBoundary.getBuilder().addValue("a").build();
    sb2 = StringBoundary.getBuilder().addValue("b").build();
    sbR = (StringBoundary) sb1.createBi(BETWEEN, sb2).get();
    assertArrayEquals(new String[]{"a", "b"}, sbR.getValues());
    assertTrue(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().addValue("a").matchesNoneOfValues(true).build();
    sb2 = StringBoundary.getBuilder().addValue("b").build();
    sbR = (StringBoundary) sb1.createBi(BETWEEN, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().addValue("a").matchesNoneOfValues(true).build();
    sb2 = StringBoundary.getBuilder().addValue("a").build();
    assertFalse(sb1.createBi(BETWEEN, sb2).isPresent());
    assertFalse(sb2.createBi(BETWEEN, sb1).isPresent());

    sb1 = StringBoundary.getBuilder().addValue("a").matchesNoneOfValues(true).build();
    sb2 = StringBoundary.getBuilder().addValue("b").matchesNoneOfValues(true).build();
    assertFalse(sb1.createBi(BETWEEN, sb2).isPresent());
    assertFalse(sb2.createBi(BETWEEN, sb1).isPresent());

    sb1 = StringBoundary.getBuilder().addValue("a").matchesNoneOfValues(true).build();
    sb2 = StringBoundary.getBuilder().addValue("a").matchesNoneOfValues(true).build();
    sbR = (StringBoundary) sb1.createBi(BETWEEN, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 =
        StringBoundary.getBuilder()
            .addValue("a")
            .addValue("b")
            .addValue("c")
            .matchesNoneOfValues(true)
            .build();
    sb2 = StringBoundary.getBuilder().addValue("a").matchesNoneOfValues(true).build();
    sbR = (StringBoundary) sb1.createBi(BETWEEN, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().addValue("a").build();
    sb2 = StringBoundary.getBuilder().addValue("a").build();
    sbR = (StringBoundary) sb1.createBi(BETWEEN, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertTrue(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());
  }
}
