package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType.INTERSECTION;
import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import io.quarkus.test.junit.QuarkusTest;
import org.junit.jupiter.api.Test;

@QuarkusTest
class BiCreaterStringIntersectionTest {

  StringBoundary sb1;
  StringBoundary sb2;
  StringBoundary sbR;

  @Test
  void testCreater() {
    sb1 = StringBoundary.getBuilder().addValue("a").build();
    sb2 = StringBoundary.getBuilder().addValue("b").build();
    assertFalse(sb1.createBi(INTERSECTION, sb2).isPresent());

    sb1 = StringBoundary.getBuilder().addValue("a").build();
    sb2 = StringBoundary.getBuilder().addValue("a").build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().matchesAny(true).build();
    sb2 = StringBoundary.getBuilder().matchesAny(true).build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertTrue(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().matchesAny(true).build();
    sb2 = StringBoundary.getBuilder().addValue("a").build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().addValue("a").build();
    sb2 = StringBoundary.getBuilder().matchesAny(true).build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().matchesAny(true).build();
    sb2 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertTrue(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").build();
    sb2 = StringBoundary.getBuilder().matchesAny(true).build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertTrue(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").build();
    sb2 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a"}, sbR.getValues());
    assertTrue(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").build();
    sb2 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").addValue("b").build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a", "b"}, sbR.getValues());
    assertTrue(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").addValue("b").build();
    sb2 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a", "b"}, sbR.getValues());
    assertTrue(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().addValue("a").addValue("b").build();
    sb2 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"b"}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().addValue("a").addValue("b").build();
    sb2 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("a").addValue("b").build();
    assertFalse(sb1.createBi(INTERSECTION, sb2).isPresent());

    sb1 = StringBoundary.getBuilder().addValue("a").addValue("b").build();
    sb2 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("c").build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a", "b"}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());

    sb1 = StringBoundary.getBuilder().matchesNoneOfValues(true).addValue("c").build();
    sb2 = StringBoundary.getBuilder().addValue("a").addValue("b").build();
    sbR = (StringBoundary) sb1.createBi(INTERSECTION, sb2).get();
    assertArrayEquals(new String[]{"a", "b"}, sbR.getValues());
    assertFalse(sbR.matchesNoneOfValues());
    assertFalse(sbR.matchesAny());
  }
}
